import unittest.mock
from typing import Dict

import requests
from nyckel import ClassificationPrediction, ClassificationPredictionError, Credentials, TagsPrediction
from nyckel.functions.tags.tags_sample_handler import TagsSampleHandler


def test_server_side_invoke_error_handling(auth_test_credentials: Credentials) -> None:

    # Mock the _post_as_json method to return empty response for "invalid_input"
    with unittest.mock.patch("nyckel.request_utils.ParallelPoster._post_as_json") as mock_post:

        def side_effect(data: Dict) -> requests.Response:
            response = requests.Response()
            if data.get("data") == "server side error":
                response.status_code = 400
                response._content = b'"Invalid input"'
                return response

            # Create successful response with JSON payload
            response.status_code = 200
            response._content = (
                b'[{"labelName": "Nice", "confidence": 0.9}, {"labelName": "Friendly", "confidence": 0.8}]'
            )
            return response

        mock_post.side_effect = side_effect
        sample_handler = TagsSampleHandler("function_id", auth_test_credentials)
        preds = sample_handler.invoke(["valid input", "server side error"], lambda x: x)
        assert len(preds) == 2
        assert len(preds[0]) == 2
        assert isinstance(preds[0][0], ClassificationPrediction)
        assert len(preds[1]) == 1
        assert isinstance(preds[1][0], ClassificationPredictionError)



from .auth import Credentials  # noqa: F401, I001
from .auth import Credentials as OAuth2Renewer  # For backwards compatibility. # noqa: F401
from .auth import Credentials as User  # For backwards compatibility. # noqa: F401
from .data_classes import NyckelId  # noqa: F401
from .functions.classification.classification import (
    ClassificationSampleSet,  # noqa: F401
    ClassificationAnnotation,  # noqa: F401
    ClassificationFunction,  # noqa: F401
    ClassificationLabel,  # noqa: F401
    ClassificationPrediction,  # noqa: F401
    ClassificationPredictionError,  # noqa: F401
    ClassificationPredictionOrError,  # noqa: F401
    ClassificationSample,  # noqa: F401
    ImageClassificationSample,  # noqa: F401
    ImageSampleData,  # noqa: F401
    LabelName,  # noqa: F401
    TabularClassificationSample,  # noqa: F401
    TabularFieldKey,  # noqa: F401
    TabularFieldValue,  # noqa: F401
    TabularFunctionField,  # noqa: F401
    TabularSampleData,  # noqa: F401
    TextClassificationSample,  # noqa: F401
    TextSampleData,  # noqa: F401
)
from .functions.tags.tags import (
    ImageTagsSample,  # noqa: F401
    TabularTagsSample,  # noqa: F401
    TagsAnnotation,  # noqa: F401
    TagsPrediction,  # noqa: F401
    TextTagsSample,  # noqa: F401
)
from .image_processing import ImageDecoder, ImageEncoder, ImageResizer  # noqa: F401
from .functions.classification.factory import ClassificationFunctionFactory  # noqa: F401
from .functions.classification.image_classification import ImageClassificationFunction  # noqa: F401
from .functions.classification.tabular_classification import TabularClassificationFunction  # noqa: F401
from .functions.classification.text_classification import TextClassificationFunction  # noqa: F401
from .functions.pretrained import invoke  # noqa: F401
from .functions.tags.image_tags import ImageTagsFunction  # noqa: F401
from .functions.tags.tabular_tags import TabularTagsFunction  # noqa: F401

from .functions.tags.text_tags import TextTagsFunction  # noqa: F401

__all__ = [
    "Credentials",
    "OAuth2Renewer",  # Backwards compatibility
    "User",  # Backwards compatibility
    "NyckelId",
    "ClassificationAnnotation",
    "ClassificationFunction",
    "ClassificationLabel",
    "ClassificationSampleSet",
    "ClassificationPrediction",
    "ClassificationPredictionError",
    "ClassificationPredictionOrError",
    "ClassificationSample",
    "ImageClassificationSample",
    "ImageSampleData",
    "LabelName",
    "TabularClassificationSample",
    "TabularFieldKey",
    "TabularFieldValue",
    "TabularFunctionField",
    "TabularSampleData",
    "TextClassificationSample",
    "TextSampleData",
    "ImageTagsSample",
    "TabularTagsSample",
    "TagsAnnotation",
    "TagsPrediction",
    "TextTagsSample",
    "ImageDecoder",
    "ImageEncoder",
    "ImageResizer",
    "ClassificationFunctionFactory",
    "ImageClassificationFunction",
    "TabularClassificationFunction",
    "TextClassificationFunction",
    "invoke",
    "ImageTagsFunction",
    "TabularTagsFunction",
    "TextTagsFunction",
]
